from django.shortcuts import render, redirect, reverse
from django.contrib.auth import authenticate, login, logout
from django.contrib import messages
from .forms import GroupForm, GroupAdminForm
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from .forms import UserForm, RoleForm, updateUserForm
from .models import Acount_User, Roles
from dashboard.models import CompanyLocation
from django.http import JsonResponse

# Create your views here.
def account_login(request):
    if request.user.is_authenticated:
        return redirect(reverse('dashboard'))
    if request.method == 'POST':
        username=request.POST['username']
        password=request.POST['password']
        
        user = authenticate(request, username=request.POST['username'], password=request.POST['password'])
        if user is not None:
            login(request, user)
            return redirect(reverse('dashboard'))
        else:
            messages.error(request, 'Invalid username or password')
    return render(request, 'accounts/login.html')


def account_logout(request):
    logout(request)
    return redirect(reverse('accounts'))




# user roles and permissions 
 
def create_user(request):
    try:
        if request.user.is_superuser and request.user.roles == None:
            pass 
        elif request.user.roles.role or request.user.is_superuser:
            pass

        else:
            messages.error(request, 'You do not have permission to access this page')
            return redirect(reverse('dashboard'))
    except:
        messages.error(request, 'You do not have permission to access this page')
        return redirect(reverse('accounts'))
    
    form = UserForm(request.POST or None)
    users = Acount_User.objects.all()
    locations = CompanyLocation.objects.all()
    roles = Roles.objects.all()
    
    if request.method == 'POST':
        form = UserForm(request.POST)
        user = request.POST.get('username')
        password = request.POST.get('password')
        
        
        if form.is_valid():
            user = form.save(commit=False)
            password = form.cleaned_data.get('password')
            user.set_password(password)
            user.save()
            
            messages.success(request, 'User created successfully')
        else:
            messages.error(request, 'User not created')
    
    
    context = {'form': form, 'users': users, 'locations': locations, 'roles': roles}
    return render(request, 'accounts/users.html', context)



def deleteUsers(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")
    user = None
    id = request.POST.get('id')
    try:
        user = Acount_User.objects.get(id=id)
    except:
        messages.error(request, 'User not deleted')
    if user:
        try:
            user.delete()
            messages.success(request, 'User deleted successfully')
        except:
            messages.error(request, 'User not deleted')
    else:
        messages.error(request, 'User not deleted')
    
    return redirect(reverse('users'))



def updateUsers(request):
    if request.method != 'POST':
        messages.error(request, "Access Denied")
    try:
        user = request.POST.get('id')
        username = request.POST.get('username')
        first_name = request.POST.get('first_name')
        last_name = request.POST.get('last_name')
        companyLocation = request.POST.get('companyLocation')
        is_superuser = request.POST.get('is_superuser')
        is_active = request.POST.get('is_active')
        
        
        user = Acount_User.objects.filter(id=user)[0] 
        
        form = updateUserForm(request.POST or None, instance=user)
        if form.is_valid():
            form.save()
            messages.success(request, "User Updated Successfully")
        else:
            messages.error(request, "User not updated")
    except:
        messages.error(request, "Access To This Resource Denied")
    
    return redirect(reverse('users'))



def roles(request):
    try:
        if request.user.is_superuser and request.user.roles == None:
            pass 
        elif request.user.roles.role or request.user.is_superuser:
            pass
        elif not request.user.is_authenticated:
            messages.error(request, 'You do not have permission to access this page')
            return redirect(reverse('dashboard'))
        else:
            messages.error(request, 'You do not have permission to access this page')
            return redirect(reverse('dashboard'))
    except:
        messages.error(request, 'You do not have permission to access this page')
        return redirect(reverse('dashboard'))
    
    form = RoleForm(request.POST or None)
    roles = Roles.objects.all()
    if request.method == 'POST':
        form = RoleForm(request.POST)
        
        if form.is_valid():
            role = form.save(commit=False)
            role.created_by = request.user
            role.save(request.user)
            
            
            messages.success(request, 'Role created successfully')
        else:
            messages.error(request, 'Role not created')
    
    
    context = {'form': form, 'roles': roles}
    return render(request, 'accounts/roles.html', context)



def viewUserById(request):
    id = request.GET.get('id', None)
    user = Acount_User.objects.filter(id=id)
    context = {}
    
    if not user.exists():
        context["code"] = 404
    else:
        context["code"] = 200
        user = user[0]
        context["id"] = user.id
        context["username"] = user.username
        context["first_name"] = user.first_name
        context["last_name"] = user.last_name
        if user.companyLocation:
            context["companyLocation"] = user.companyLocation.id
        context["phone"] = user.phone
        if user.roles:
            context["role"] = user.roles.id
        context["is_active"] = user.is_active
        context["is_superuser"] = user.is_superuser
        #print(context)
        
        return JsonResponse(context)
    
    
    
def viewRolesById(request):
    id = request.GET.get('id', None)
    roles = Roles.objects.filter(id=id)
    context = {}  
      
    if not roles.exists():
        context["code"] = 404
    else:
        context["code"] = 200
        role = roles[0]
        context["id"] = role.id
        context["name"] = role.name
        context["company_location"] = role.company_location
        context["company_profile"] = role.company_profile
        context["product_category"] = role.product_category
        context["products"] = role.products
        context["product_prices"] = role.product_prices
        context["product_stocks"] = role.product_stocks
        context["customers"] = role.customers
        context["suppliers"] = role.suppliers
        context["role"] = role.role
        context["bank_accounts"] = role.bank_accounts
        context["users"] = role.users
        
        context["package"] = role.package
        context["expense_type"] = role.expense_type
        context["bank"] = role.bank
        context["payment_method"] = role.payment_method
        context["bank_transaction_type"] = role.bank_transaction_type
        
        context["cash_sale"] = role.cash_sale
        context["credit_sale"] = role.credit_sale
        context["stock_purchase"] = role.stock_purchase
        context["stock_transfer"] = role.stock_transfer
        context["protocol"] = role.protocol
        context["expense"] = role.expense
        context["bank_transaction"] = role.bank_transaction
        context["can_print_sales"] = role.can_print_sales
        context["can_return_sales"] = role.can_return_sales
        
        
        context["credit_sale_payment"] = role.credit_sale_payment
        context["stock_purchase_payment"] = role.stock_purchase_payment
        
        context["sale_return"] = role.sale_return
        context["stock_purchase_return"] = role.stock_purchase_return
        context["stock_transfer_return"] = role.stock_transfer_return
        context["protocol_return"] = role.protocol_return
        context["stock_damage_return"] = role.stock_damage_return
        context["stock_purchase_update"] = role.stock_purchase_update
        context["sale_update"] = role.sale_update
        
        
        context["sale_report"] = role.sale_report
        context["sale_summary_report"] = role.sale_summary_report
        context["customer_account_report"] = role.customer_account_report
        context["customer_statement_report"] = role.customer_statement_report
        context["supplier_account_report"] = role.supplier_account_report
        context["supplier_statement_report"] = role.supplier_statement_report
        context["product_sale_report"] = role.product_sale_report
        context["stock_value_report"] = role.stock_value_report
        context["stock_control_report"] = role.stock_control_report
        context["stock_purchase_report"] = role.stock_purchase_report
        context["stock_transfer_report"] = role.stock_transfer_report
        context["protocol_report"] = role.protocol_report
        context["product_purchase_report"] = role.product_purchase_report
        context["transaction_return_report"] = role.transaction_return_report
        context["user_activity_report"] = role.user_activity_report
        context["expenses_report"] = role.expenses_report
        context["bank_transaction_report"] = role.bank_transaction_report
        context["customer_performance_report"] = role.customer_performance_report
        context["stock_control_summary_report"] = role.stock_control_summary_report
        
        #print(context['id'])
        
    return JsonResponse(context)


def deleteRole(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")
    role = None
    id = request.POST.get('id')
    try:
        role = Roles.objects.get(id=id)
    except:
        messages.error(request, 'Role not deleted')
    if role:
        try:
            role.delete()
            messages.success(request, 'Role deleted successfully')
        except:
            messages.error(request, 'Role not deleted')
    else:
        messages.error(request, 'Role not deleted')
    
    return redirect(reverse('roles'))




def updateRoles(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")
    id = request.POST.get('id')
    try:
        role = Roles.objects.filter(id=id)[0]
    except:
        messages.error(request, 'Role not updated')
    
    form = RoleForm(request.POST, instance=role)

    if form.is_valid():
        form.save()
        messages.success(request, 'Role updated successfully')
    else:
        messages.error(request, 'Role not updated')
    
    return redirect(reverse('roles'))




def user_role_update(request, id):
    
    role = Roles.objects.get(id=id)
    form = RoleForm(instance=role)
    
    if request.method == 'POST':
        form = RoleForm(request.POST, instance=role)
        if form.is_valid():
            form.save()
            return redirect(reverse('roles'))
    
    
    
    return render(request, 'accounts/editRole.html', {'form': form, 'role': role})






def reset_password(request):

    if request.method == 'POST':
        password = request.POST.get("password")
        if len(password) < 5:
            messages.error(request, "Password much be 5 or more")
            return redirect(reverse("reset_password"))
        user = Acount_User.objects.get(id=request.user.id)
        user.set_password(password)
        user.save()
        messages.success(request, "Password Reset Successfull")    
    
    return render(request, 'accounts/reset_password.html')